<?php
/*!
    \brief   posixAccount plugin
    \author  Cajus Pollmeier <pollmeier@gonicus.de>
    \version 2.00
    \date    24.07.2003

    This class provides the functionality to read and write all attributes
    relevant for posixAccounts and shadowAccounts from/to the LDAP. It
    does syntax checking and displays the formulars required.
 */

class posixAccount extends plugin
{
  /* Plugin specific values */
  var $homeDirectory= "";
  var $loginShell= "/bin/bash";
  var $uidNumber= "";
  var $gidNumber= "";
  var $gecos= "";
  var $shadowMin= "0";
  var $shadowMax= "0";
  var $shadowWarning= "0";
  var $shadowLastChange= "0";
  var $shadowInactive= "0";
  var $shadowExpire= "0";
  var $gosaDefaultPrinter= "";
  var $gosaDefaultLanguage= "";
  var $gosaHostACL= array();

  var $status= "";
  var $loginShellList= array();
  var $groupMembership= array();
  var $savedGroupMembership= array();
  var $savedUidNumber= "";
  var $savedGidNumber= "";
  var $use_shadowMin= "0";
  var $use_shadowMax= "0";
  var $use_shadowWarning= "0";
  var $use_shadowInactive= "0";
  var $use_shadowExpire= "0";
  var $must_change_password= "0";
  var $force_ids= 0;
  var $printerList= array();
  var $group_dialog= FALSE;
  var $hosts_dialog= FALSE;
  var $shadow_dialog= FALSE;
  var $secondaryGroups= array();
  var $primaryGroup= 0;


  /* attribute list for save action */
  var $attributes= array("homeDirectory", "loginShell", "uidNumber", "gidNumber", "gecos",
			"shadowMin", "shadowMax", "shadowWarning", "shadowInactive", "shadowLastChange",
			"shadowExpire", "gosaDefaultPrinter", "gosaDefaultLanguage", "uid");
  var $objectclasses= array("posixAccount", "shadowAccount");


  /* constructor, if 'dn' is set, the node loads the given
     'dn' from LDAP */
  function posixAccount ($config, $dn= NULL)
  {
        /* Configuration is fine, allways */
        $this->config= $config;

	/* Load bases attributes */
	plugin::plugin($config, $dn);

	if ($dn != NULL){
		/* Assemble host acl's */
		if (isset($this->attrs['gosaHostACL'][0])){
			for ($i= 0; $i<$this->attrs['gosaHostACL']['count']; $i++){
				$name= preg_replace ("/:.*$/", "", $this->attrs['gosaHostACL'][$i]);
				$description= rtrim(preg_replace ("/^[^:]+:/", "", $this->attrs['gosaHostACL'][$i]), ':');
				$description= preg_replace ('/:/', ',', $description);
				$this->gosaHostACL[$name]= $description;
			}
		}

		/* Fill group */
		$this->primaryGroup= $this->gidNumber;

		/* Generate status text */
		$current= date("U") / 86400;
		if (($current >= $this->shadowExpire) && $this->shadowExpire){
			$this->status= "expired";
			if (($this->shadowExpire - $current) < $this->shadowInactive){
				$this->status.= ", grace time active";
			}
		} elseif (($this->shadowLastChange + $this->shadowMin) >= $current){
			$this->status= "active, password not changable";
		} elseif (($this->shadowLastChange + $this->shadowMax) >= $current){
			$this->status= "active, password expired";
		} else {
			$this->status= "active";
		}

		/* Get group membership */
		$ldap= $this->config->get_ldap_link();
		$ldap->cd($this->config->current['BASE']);
		$ldap->search("(&(objectClass=posixGroup)(memberUid=".$this->uid."))");

		while ($this->attrs= $ldap->fetch()){
			if (!isset($this->attrs["description"][0])){
			$entry= "[".$this->attrs["cn"][0]."]";
		} else {
			$entry= utf8_decode($this->attrs["description"][0]).
				" [".$this->attrs["cn"][0]."]";
		}
			$this->groupMembership[$ldap->getDN()]= $entry;
		}
		$this->savedGroupMembership= $this->groupMembership;
		$this->savedUidNumber= $this->uidNumber;
		$this->savedGidNumber= $this->gidNumber;
	}

	/* Adjust shadow checkboxes */
	foreach (array("shadowMin", "shadowMax", "shadowWarning", "shadowInactive",
		"shadowExpire") as $val){

		if ($this->$val != 0){
			$oval= "use_".$val;
			$this->$oval= "1";
		}
	}

	/* Generate shell list from /etc/shells */
	$shells = file ('/etc/shells');
	foreach ($shells as $line){
		if (!preg_match ("/^#/", $line)){
			$this->loginShellList[]= trim($line);
		}
	}

	/* Generate printer list */
	if (isset($this->config->data['SERVERS']['CUPS'])){
		$this->printerList= get_printer_list ($this->config->data['SERVERS']['CUPS']);
		asort($this->printerList);
	}

	/* Generate group list */
	$ldap= $this->config->get_ldap_link();
	$ldap->cd($this->config->current['BASE']);
	$ldap->search("(objectClass=posixGroup)");
	$this->secondaryGroups[]= "- "._("automatic")." -";
	while ($attrs= $ldap->fetch()){
		$this->secondaryGroups[$attrs['gidNumber'][0]]= $attrs['cn'][0];
	}
	asort ($this->secondaryGroups);
  }


  /* execute generates the html output for this node */
  function execute()
  {
        /* Do we need to flip is_account state? */
        if (isset($_POST['modify_state'])){
                $this->is_account= !$this->is_account;
        }

	/* Get ldap link */
	$ldap= $this->config->get_ldap_link();

	/* Check sorting variable */
	$this->check_sortranges();

	/* Do we represent a valid posixAccount? */
	if (!$this->is_account && $this->parent == NULL ){
		echo "<img src=\"images/stop.png\" align=center>&nbsp;<b>".
			_("This account has no valid unix extensions.")."</b>";
		return;
	}

	/* Show tab dialog headers */
	if ($this->parent != NULL){
		if ($this->is_account){
			if (isset($this->parent->by_object['sambaAccount'])){
				$obj= $this->parent->by_object['sambaAccount'];
			}
			if (isset($obj) && $obj->is_account == TRUE &&
				isset($this->parent->by_object['sambaAccount'])){
				/* Samba3 dependency on posix accounts are enabled
				   in the moment, because I need to rely on unique
				   uidNumbers. There'll be a better solution later
				   on. */
				# && $this->config->current['SAMBAVERSION'] != 3){

				$this->show_header(_("Remove posix account"),
					_("This account has unix features enabled. ".
					  "To disable them, you'll need to remove ".
					  "the samba account first."), TRUE);
			} else {
				$this->show_header(_("Remove posix account"),
					_("This account has posix features enabled. ".
					  "You can disable them by clicking below."));
			}
		} else {
			$this->show_header(_("Create posix account"),
				_("This account has posix features disabled. ".
				  "You can enable them by clicking below."));
			return;
		}
	}

	/* Trigger group edit? */
	if (isset($_POST['edit_groupmembership'])){
		$this->group_dialog= TRUE;
		$this->dialog= TRUE;
	}

	/* Cancel group/hosts edit? */
	if (isset($_POST['add_groups_cancel']) ||
		isset($_POST['add_groups_finish']) ||
		isset($_POST['add_hosts_cancel']) ||
		isset($_POST['add_hosts_finish']) ||
		isset($_POST['edit_shadow_finish']) ||
		isset($_POST['edit_shadow_cancel'])){
		$this->group_dialog= FALSE;
		$this->hosts_dialog= FALSE;
		$this->shadow_dialog= FALSE;
		$this->dialog= FALSE;
	}

	/* Add selected groups */
	if (isset($_POST['add_groups_finish']) && isset($_POST['groups']) &&
		count($_POST['groups'])){

		if (chkacl ($this->acl, "memberUid") == ""){
			$this->addGroup ($_POST['groups']);
		}
	}

	/* Delete selected groups */
	if (isset($_POST['delete_groupmembership']) && 
		isset($_POST['group_list']) && count($_POST['group_list'])){

		if (chkacl ($this->acl, "memberUid") == ""){
			$this->delGroup ($_POST['group_list']);
		}
	}

	/* Manage group add dialog */
	if ($this->group_dialog){

		/* Calculate actual groups */
		$groups= array ();
		if (isset ($_POST['department'])){
			$ldap->cd ($this->config->departments[$_POST['department']]);
		} else {
			$ldap->cd ($this->config->current['BASE']);
		}
		$ldap->search ("(objectClass=posixGroup)");

		while ($attrs= $ldap->fetch()){
			if (preg_match ($this->sortranges[$this->sorting], $attrs["cn"][0])){
				if (!array_key_exists($ldap->getDN(), $this->groupMembership)){
					error_reporting (0);
					$groups[$ldap->getDN()]= utf8_decode($attrs['description'][0])." [".
									$attrs['cn'][0]."]";
					error_reporting (E_ALL);
				}
			}
		}
		ksort ($groups);
		reset ($groups);

		/* Show dialog */
		require_once ("edit_groups.inc");
		return;
	}

	/* Shadow dialog */
	if (isset($_POST['edit_shadow'])){
		$this->shadow_dialog= TRUE;
		$this->dialog= TRUE;
		require_once ("edit_shadow.inc");
		return;
	}

	/* Apply shadow dialog */
	if (isset($_POST['edit_shadow_finish'])){
		foreach (array("must_change_password", "use_shadowMin", "use_shadowMax",
				"use_shadowWarning", "use_shadowInactive", "use_shadowExpire") as $val) {

			if (chkacl ($this->acl, "$val") == "" && isset ($_POST["$val"])){
				$this->$val= $_POST["$val"];
			} else {
				$this->$val= "";
			}
		}
	}

	/* Update shadowExpire value */
	if (chkacl ($this->acl, "shadowExpire") == "" && isset ($_POST['date_month'])){
		$this->shadowExpire= (int)(mktime(0,0,0,$_POST['date_month'],
							$_POST['date_day']+1,
							$_POST['date_year']) / 86400);
	}



	
	/* Trigger group edit? */
	if (isset($_POST['edit_hostacls'])){
		$this->hosts_dialog= TRUE;
		$this->dialog= TRUE;
	}

	/* Manage hostacl dialog */
	if ($this->hosts_dialog){

		$hosts= array ();
		$ldap->search ("(objectClass=gosaHost)");
		while ($attrs= $ldap->fetch()){
			if (preg_match ($this->sortranges[$this->sorting], $attrs["cn"][0])){
				$hosts[$ldap->getDN()]= $attrs['cn'][0];
			}
		}
		ksort ($hosts);
		reset ($hosts);

		/* Show dialog */
		require_once ("edit_hosts.inc");
		return;
	}
	

	/* Show main page */
	require ("edit.inc");
  }


  /* remove object from parent */
  function remove_from_parent()
  {
	/* include global link_info */
	$ldap= $this->config->get_ldap_link();

	/* Remove and write to LDAP */
	plugin::remove_from_parent();

	/* Zero out array */
	$this->attrs['gosaHostACL']= array();

	/* Keep uid, because we need it for authentification! */
	unset($this->attrs['uid']);

        @DEBUG (DEBUG_LDAP, __LINE__, __FUNCTION__, __FILE__,
                $this->attributes, "Save");
	$ldap->cd($this->dn);
	$ldap->modify($this->attrs);
	show_ldap_error($ldap->error);

	/* Delete group only if cn is uid and there are no other
	   members inside */
	$ldap->cd ($this->config->current['BASE']);
	$ldap->search ("(&(objectClass=posixGroup)(gidNumber=".$this->gidNumber."))");
	if ($ldap->count() != 0){
		$attrs= $ldap->fetch();
		if ($attrs['cn'][0] == $this->uid &&
			!isset($this->attrs['memberUid'])){

			$ldap->rmDir($ldap->getDN());
		}
	}

	/* Delete references */
	$ldap->cd ($this->config->current['BASE']);
	$ldap->search ("(&(objectClass=posixGroup)(memberUid=".$this->uid."))");
	while ($ldap->fetch()){
		$g= new group($this->config, $ldap->getDN());
		$g->removeUser($this->uid);
		$g->save ();
	}

	/* Optionally execute a command after we're done */
	$this->postremove();
  }


  function save_object()
  {
	if (isset($_POST['posixTab'])){
		/* Save values to object */
		plugin::save_object();

		/* Save force GID attribute */
		if (chkacl ($this->acl, "force_ids") == ""){
			if (isset ($_POST['force_ids'])){
				$this->force_ids= 1;
			} else {
				$this->force_ids= 0;
			}
			$this->primaryGroup= $_POST['primaryGroup'];
		}
	}
  }


  /* Save data to LDAP, depending on is_account we save or delete */
  function save()
  {
	/* include global link_info */
	$ldap= $this->config->get_ldap_link();

	/* Adapt shadow values */
	if (!$this->use_shadowExpire){
		$this->shadowExpire= 0;
	}
	if ($this->must_change_password){
		$this->shadowLastChange= (int)(date("U") / 86400) - $this->shadowMax - 1;
	} else {
		$this->shadowLastChange= (int)(date("U") / 86400);
	}

	/* Fill gecos */
	$this->gecos= $this->uid;

	/* Check what to do with ID's */
	if ($this->force_ids == 0){

		/* Use id's that are already set */
		if ($this->savedUidNumber != ""){
			$this->uidNumber= $this->savedUidNumber;
			$this->gidNumber= $this->savedGidNumber;
		} else {

			/* Calculate new id's. We need to place a lock before calling get_next_id
			   to get real unique values. */
			$wait= 10;
			while (get_lock("uidnumber") != ""){
				sleep (1);
	
				/* Oups - timed out */
				if ($wait-- == 0){
					print_red (_("Failed: overriding lock"));
					break;
				}
			}

			add_lock ("uidnumber", "gosa");
			$this->uidNumber= $this->get_next_id("uidNumber");
			$this->gidNumber= $this->get_next_id("gidNumber");
		}

		if ($this->primaryGroup != 0){
			$this->gidNumber= $this->primaryGroup;
		}
	}

	/* Call parents save to prepare $this->attrs */
	plugin::save();
	
	/* Add support for host acl's */
	if (is_array($this->gosaHostACL)){
		foreach ($this->gosaHostACL as $key => $value){
			$this->attrs["gosaHostACL"][]= "$key:".preg_replace("/,/", ":", $value).":";
		}
	}

	/* Save data to LDAP */
	$ldap->cd($this->dn);
	$ldap->modify($this->attrs);
	show_ldap_error($ldap->error);

	/* Remove lock needed for unique id generation */
	del_lock ("uidnumber");

	
	/* Posix accounts have group interrelationship, take care about these here. */
	if ($this->force_ids == 0 && $this->primaryGroup == 0){
		$name= preg_replace ("/,ou=people.*/i", "", $this->dn);
		$name= preg_replace ("/^cn=/i", "", $name);
		$groupdn= preg_replace ("/^cn=[^,]+,ou=people,/i", "cn=$this->uid,ou=groups,", $this->dn);
		$ldap->cat($groupdn);
		$this->attrs= $ldap->fetch();

		/* Create group if it doesn't exist */
		if (!isset($this->attrs['cn'][0])){
			$g= new group($this->config, $groupdn);
			$g->cn= $this->uid;
			$g->force_gid= 1;
			$g->gidNumber= $this->gidNumber;
			$g->description= _("Group of user")." ".utf8_decode($name);
			$g->save ();
		}
	}

	/* Take care about groupMembership values: add to groups */
	foreach ($this->groupMembership as $key => $value){
		$g= new group($this->config, $key);
		$g->addUser ($this->uid);
		$g->save();
	}

	/* Remove from groups not listed in groupMembership */
	foreach ($this->savedGroupMembership as $key => $value){
		if (!array_key_exists ($key, $this->groupMembership)){
		        $g= new group($this->config, $key);
		        $g->removeUser ($this->uid);
		        $g->save();
	      }
	}

	/* Optionally execute a command after we're done */
	$this->postcreate();
  }

  /* Check formular input */
  function check()
  {
	/* Include global link_info */
	$ldap= $this->config->get_ldap_link();

	/* Reset message array */
	$message= array();

	/* must: homeDirectory */
	if ($this->homeDirectory == ""){
		$message[]= _("The required field 'Home directory' is not set.");
	}
	if (!is_path($this->homeDirectory)){
		$message[]= _("Please enter a valid path in 'Home directory' field.");
	}

	/* Check ID's if they are forced by user */
	if ($this->force_ids == "1"){

		/* Valid uid/gid? */
		if (!is_id($this->uidNumber)){
			$message[]= _("Value specified as 'UID' is not valid.");
		}
		if (!is_id($this->gidNumber)){
			$message[]= _("Value specified as 'GID' is not valid.");
		}
	}

	/* Check shadow settings, well I like spaghetties... */
	if ($this->use_shadowMin){
		if (!is_id($this->shadowMin)){
			$message[]= _("Value specified as 'shadowMin' is not valid.");
		}
	}
	if ($this->use_shadowMax){
		if (!is_id($this->shadowMax)){
			$message[]= _("Value specified as 'shadowMax' is not valid.");
		}
	}
	if ($this->use_shadowWarning){
		if (!is_id($this->shadowWarning)){
			$message[]= _("Value specified as 'shadowWarning' is not valid.");
		}
		if (!$this->use_shadowMax){
			$message[]= _("'shadowWarning' without 'shadowMax' makes no sense.");
		}
		if ($this->shadowWarning > $this->shadowMax){
			$message[]= _("Value specified as 'shadowWarning' should be smaller than 'shadowMax'.");
		}
		if ($this->use_shadowMin && $this->shadowWarning < $this->shadowMin){
			$message[]= _("Value specified as 'shadowWarning' should be greater than 'shadowMin'.");
		}
	}
	if ($this->use_shadowInactive){
		if (!is_id($this->shadowInactive)){
			$message[]= _("Value specified as 'shadowInactive' is not valid.");
		}
		if (!$this->use_shadowMax){
			$message[]= _("'shadowInactive' without 'shadowMax' makes no sense.");
		}
	}
	if ($this->use_shadowMin && $this->use_shadowMax){
		if ($this->shadowMin > $this->shadowMax){
			$message[]= _("Value specified as 'shadowMin' should be smaller than 'shadowMax'.");
		}
	}

	return ($message);
  }


  /* Add posix user to some groups */
  function addGroup ($groups)
  {
	/* include global link_info */
	$ldap= $this->config->get_ldap_link();

	/* Walk through groups and add the descriptive entry if not exists */
	foreach ($groups as $value){
		if (!array_key_exists($value, $this->groupMembership)){
			$ldap->cat($value);
			$attrs= $ldap->fetch();
			error_reporting (0);
			$entry= utf8_decode($attrs["description"][0])." [".$attrs["cn"][0]."]";
			error_reporting (E_ALL);
			$this->groupMembership[$ldap->getDN()]= $entry;
		}
	}

	/* Sort groups */
	asort ($this->groupMembership);
	reset ($this->groupMembership);
  }


  /* Del posix user from some groups */
  function delGroup ($groups)
  {
	$dest= array();

	foreach ($this->groupMembership as $key => $value){
		if (!in_array($key, $groups)){
			$dest[$key]= $value;
		}
	}
	$this->groupMembership= $dest;
  }

  /* Adapt from template, using 'dn' */
  function adapt_from_template($dn)
  {
	/* Include global link_info */
	$ldap= $this->config->get_ldap_link();

	plugin::adapt_from_template($dn);

	/* Adapt hostACL's without exchanging uid/sn/etc. */
	$this->gosaHostACL= array();
	if (isset($this->attrs["gosaHostACL"]["count"])){
		for ($i= 0; $i<$this->attrs["gosaHostACL"]["count"]; $i++){
			$value= utf8_decode($this->attrs["gosaHostACL"][$i]);
			array_push($this->gosaHostACL, $value);
		}
	}

	/* Adapt group membership */
	$ldap->cd($this->config->current['BASE']);
	$ldap->search("(&(objectClass=posixGroup)(memberUid=".$this->attrs['uid']."))");

	while ($this->attrs= $ldap->fetch()){
		if (!isset($this->attrs["description"][0])){
			$entry= "[".$this->attrs["cn"][0]."]";
		} else {
			$entry= $this->attrs["description"][0]." [".$this->attrs["cn"][0]."]";
		}
		$this->groupMembership[$ldap->getDN()]= $entry;
	}
  }

  function get_next_id($attrib)
  {
	$ids= array();
	$ldap= $this->config->get_ldap_link();

	$ldap->cd ($this->config->current['BASE']);
	$ldap->search ("($attrib=*)");

	# Get list of ids
	while ($attrs= $ldap->fetch()){
		$ids[]= (int)$attrs["$attrib"][0];
	}

	# Find out next free id near to UID_BASE
	for ($id= $this->config->current['UIDBASE']; $id++; $id<65000){
		if (!in_array($id, $ids)){
			return ($id);
		}
	}

	# Should not happen
	if ($id == 65000){
		printe ("get_next_id", "Congratulations, you've too much users: No free ID found");
	}

}


}
?>
